@@ -486,16 +486,6 @@ class Client():
         self.batch_span: debugging.Span|None = None
         self.last_database_version:int|None = None
 
-        # Currently, there is issue that causes Tasks to fail if their WHEN clause exercises privileges of the secondary role.
-        # This bug prevents to create data streams, which is getting quarantined due to task failure.
-        # This issue is being communicated with SF support.
-        # This is a workaround to disallow secondary role for the session.
-        # In this case, it is possible to fail fast when primary role doesn't have an access
-        # See thread: https://relationalai.slack.com/archives/C04T0R0GLR5/p1742495337255009
-        if config.get("platform", "") == "snowflake":
-            query_to_disallow_secondary_roles = "USE SECONDARY ROLES NONE"
-            self.resources._exec(query_to_disallow_secondary_roles)
-
         if not dry_run:
             if not self.use_graph_index:
                 self.create_database(isolated=isolated)
