@@ -123,10 +123,7 @@ class ModelToSQL:
         ranks = cast(list[ir.Rank], filter_by_type(task.body, ir.Rank))
         aggs = cast(list[ir.Aggregate], filter_by_type(task.body, ir.Aggregate))
         unions = cast(list[ir.Union], filter_by_type(task.body, ir.Union))
-        var_to_rank = {
-            r.result: r
-            for r in ranks
-        } if ranks else {}
+
         var_to_construct = {c.id_var: c for c in constructs} if constructs else {}
 
         statements = []
@@ -172,17 +169,18 @@ class ModelToSQL:
                             # After flatten it can be only one aggregation per rule.
                             insert = sql.Insert(self._relation_name(r), [self._var_name(r.id, f) for f in r.fields], [],
                                                 self._make_agg_select(u, lookups, aggs[0], nots, unions, constructs))
+                        elif ranks:
+                            # After flatten it can be only one rank per rule.
+                            insert = sql.Insert(self._relation_name(r), [self._var_name(r.id, f) for f in r.fields], [],
+                                                self._make_rank_select(u, lookups, ranks[0], nots, unions, constructs))
                         else:
                             aliases = []
-                            is_rank = False
                             for i, arg in enumerate(u.args):
-                                is_rank |= arg in var_to_rank
                                 field_name = self._var_name(r.id, r.fields[i])
-                                aliases.append(self._get_alias(field_name, arg, var_to_construct, var_to_rank))
+                                aliases.append(self._get_alias(field_name, arg, var_to_construct))
 
                             insert = sql.Insert(self._relation_name(r), [self._var_name(r.id, f) for f in r.fields],
-                                                [], self._make_select(lookups, aliases, nots, unions, constructs,
-                                                                      False if is_rank else True))
+                                                [], self._make_select(lookups, aliases, nots, unions, constructs, True))
                         statements.append(insert)
             elif outputs:
                 # output a query: SELECT ... FROM ... WHERE ...
@@ -191,12 +189,29 @@ class ModelToSQL:
                 for output in outputs:
                     distinct = distinct or output.keys is None
                     for key, arg in output.aliases:
-                        aliases.append(self._get_alias(key, arg, var_to_construct, var_to_rank))
+                        aliases.append(self._get_alias(key, arg, var_to_construct))
 
                 statements.append(self._make_select(lookups, aliases, nots, unions, constructs, distinct))
         elif logicals:
             for logical in logicals:
                 statements.extend(self._create_statement(logical))
+        elif not updates and not outputs:
+            # Example:
+            #   QB:
+            #       (
+            #         where(Person.age >= 65).define(Senior(Person)) |
+            #         where(Person.age >= 18).define(Adult(Person)) |
+            #         define(Child(Person))
+            #       )
+            #   After `flatten` IR will look like this:
+            #       Logical
+            #           Union
+            #               _match_7(person_7)
+            #               _match_8(person_7)
+            #               _match_9(person_7)
+            #
+            # Nothing to query or define, we need to skip this task.
+            return statements
         else:
             raise Exception(f"Cannot create SQL statement for:\n{task}")
         return statements
@@ -282,6 +297,83 @@ class ModelToSQL:
 
         return sql.Select(False, outputs, sub_select, group_by=group_by)
 
+    def _make_rank_select(self, update: ir.Update, lookups: list[ir.Lookup], rank: ir.Rank,
+                         nots: Optional[list[ir.Not]] = None, unions: Optional[list[ir.Union]] = None,
+                         constructs: Optional[list[ir.Construct]] = None):
+
+        """
+        Generate a SQL SELECT for a rank using a DISTINCT subquery.
+
+        Example output:
+            SELECT
+                cat, name, ROW_NUMBER() OVER ( ORDER BY name ASC ) as v
+            FROM (
+                SELECT DISTINCT
+                    v0.cat, v1.name
+                FROM
+                    Cat AS v0, cat_name AS v1
+                WHERE
+                    v0.cat = v1.cat
+            ) LIMIT 10;
+
+        Rationale:
+        In the IR, it’s not always explicit whether rank should be applied over distinct rows.
+        By wrapping the rank in a DISTINCT subquery, we ensure correctness regardless of whether
+        the original query used `rank(...)` or `rank(distinct ...)`.
+
+        Compare:
+
+        QB: select(rank(Cat.name))
+        IR:
+            Logical
+                Cat(cat_5)
+                name(cat_5, name)
+                rank([cat_5], [], [name'↑'], v)
+                -> derive _rank_1(cat_5, name, v)
+
+        QB: select(rank(distinct(Cat.name)))
+        IR:
+            Logical
+                Cat(cat_5)
+                name(cat_5, name)
+                rank([], [], [name'↑'], v)
+                -> derive _rank_1(name, v)
+
+        Note:
+        The key difference is that in the `distinct` case, the grouping variable `cat_5` is absent from the projection.
+        The subquery pattern unifies both cases by projecting all rank arguments, ensuring correctness.
+        """
+
+        seen_args = set()
+        outputs: list[Union[sql.VarRef, sql.RowNumberVar, int]] = []
+        sub_query_outputs: list[Tuple[Optional[str], ir.Value] | Tuple[Optional[str], ir.Value, ir.Task]] = []
+
+        order_by_vars = []
+        for arg, is_ascending in zip(rank.args, rank.arg_is_ascending):
+            order_by_vars.append(sql.OrderByVar(arg.name, is_ascending))
+        partition_by_vars = [arg.name for arg in rank.group] if rank.group else []
+
+        relation = update.relation
+
+        for i, arg in enumerate(update.args):
+            if arg not in seen_args:
+                field_name = self._var_name(relation.id, relation.fields[i])
+                if isinstance(arg, ir.Var) and arg == rank.result:
+                    outputs.append(sql.RowNumberVar(order_by_vars, partition_by_vars, field_name))
+                else:
+                    outputs.append(sql.VarRef(field_name))
+                sub_query_outputs.append((field_name, arg))
+                seen_args.add(arg)
+
+        for arg in rank.projection:
+            if arg not in seen_args:
+                sub_query_outputs.append((None, arg))
+                seen_args.add(arg)
+
+        sub_select = self._make_select(lookups, sub_query_outputs, nots, unions, constructs, True)
+
+        return sql.Select(False, outputs, sub_select, limit=rank.limit)
+
     def _make_select(self, lookups: list[ir.Lookup], outputs: list[Tuple[Optional[str], ir.Value]|Tuple[Optional[str], ir.Value, ir.Task]],
                      nots: Optional[list[ir.Not]] = None, unions: Optional[list[ir.Union]] = None,
                      constructs: Optional[list[ir.Construct]] = None, distinct: bool = False) -> sql.Select:
@@ -302,7 +394,7 @@ class ModelToSQL:
         wheres.extend(self._generate_where_clauses(var_lookups, var_column, sql_vars, builtin_vars, union_lookups,
                                                    var_to_construct))
 
-        not_null_vars, vars, limit = self._generate_select_output(outputs, builtin_vars, sql_vars, var_column,
+        not_null_vars, vars = self._generate_select_output(outputs, builtin_vars, sql_vars, var_column,
                                                                   var_lookups, var_to_construct)
 
         if not_null_vars:
@@ -313,7 +405,7 @@ class ModelToSQL:
 
         where = self._process_wheres_clauses(wheres)
 
-        return sql.Select(distinct, vars, froms, where, limit=limit)
+        return sql.Select(distinct, vars, froms, where)
 
     def _extract_lookups_metadata(self, lookups: OrderedSet[ir.Lookup], start_index: int = 0):
         froms: list[sql.From] = []
@@ -436,6 +528,10 @@ class ModelToSQL:
                     left = self._var_to_expr(lhs_raw, reference, resolve_builtin_var, var_to_construct)
                     if relation == builtins.num_chars and isinstance(rhs_raw, ir.Var):
                         builtin_vars[rhs_raw] = f"length({left})"
+                    elif relation == builtins.lower and isinstance(rhs_raw, ir.Var):
+                        builtin_vars[rhs_raw] = f"lower({left})"
+                    elif relation == builtins.upper and isinstance(rhs_raw, ir.Var):
+                        builtin_vars[rhs_raw] = f"upper({left})"
                     else:
                         right = self._var_to_expr(rhs_raw, reference, resolve_builtin_var, var_to_construct)
                         if relation == builtins.starts_with:
@@ -644,8 +740,6 @@ class ModelToSQL:
 
         # finally, compute what the select will return
         vars = []
-        # Following flattening, each rule is expected to contain at most one RANK or LIMIT clause.
-        limit: Optional[int] = None
         not_null_vars = ordered_set()
         for output in outputs:
             alias, var = output[0], output[1]
@@ -668,14 +762,6 @@ class ModelToSQL:
                     if isinstance(task, ir.Construct):
                         # Generate constructions like hash(`x`, `y`, TABLE_ALIAS.COLUMN_NAME) as `alias`
                         vars.append(sql.VarRef(handle_construct(task), alias=alias))
-                    elif isinstance(task, ir.Rank):
-                        assert alias
-                        limit = task.limit
-                        order_by_vars = []
-                        for arg, is_ascending in zip(task.args, task.arg_is_ascending):
-                            order_by_vars.append(sql.OrderByVar(reference(arg), is_ascending))
-                        partition_by_vars = [reference(arg) for arg in task.group] if task.group else []
-                        vars.append(sql.RowNumberVar(order_by_vars, partition_by_vars, alias))
                     elif isinstance(task, ir.Aggregate):
                         result_arg = task.projection[-1] if task.aggregation == builtins.count else task.args[0]
                         result_arg = resolve_builtin_var(result_arg)
@@ -685,7 +771,7 @@ class ModelToSQL:
                 # TODO - abusing even more here, because var is a value!
                 value = self._convert_value(var, False)
                 vars.append(sql.VarRef(str(value), alias=alias))
-        return not_null_vars, vars, limit
+        return not_null_vars, vars
 
     def _generate_select_nots(self, nots: Optional[list[ir.Not]], sql_vars: dict[ir.Lookup, str],
                               var_column:dict[Tuple[ir.Var, ir.Lookup], ir.Field], index: int) -> tuple[list[sql.NotExists], int]:
@@ -795,11 +881,11 @@ class ModelToSQL:
         resolved = resolve_builtin_var(var)
         return reference(resolved) if isinstance(resolved, ir.Var) else str(resolved)
 
-    def _get_alias(self, key, arg, var_to_construct, var_to_rank):
+    def _get_alias(self, key, arg, var_to_construct):
         if not isinstance(arg, ir.Var):
             return key, arg
 
-        var_task = var_to_construct.get(arg) or var_to_rank.get(arg)
+        var_task = var_to_construct.get(arg)
         return (key, arg, var_task) if var_task else (key, arg)
 
     def _get_tuples(self, logical: ir.Logical, u: ir.Update):
