@@ -90,19 +90,24 @@ def _translate_to_decls(ctx: TranslationCtx, rule: ir.Logical) -> list[lqp.Decla
     return [_translate_effect(ctx, effect, mk_and(conjuncts)) for effect in effects]
 
 def _translate_effect(ctx: TranslationCtx, effect: Union[ir.Output, ir.Update], body: lqp.Formula) -> lqp.Declaration:
-    # Handle the bindings
     bindings = _effect_bindings(effect)
-    projection, eqs = translate_bindings(ctx, bindings)
+    if isinstance(effect, ir.Output):
+        projection, eqs, suffix = _translate_output_bindings(ctx, bindings)
+        meta_id = effect.id
+        def_name = "output" + suffix
+        # Uniquify the output name
+        def_name = ctx.def_names.get_name_by_id(meta_id, def_name)
+    else:
+        projection, eqs = _translate_bindings(ctx, bindings)
+        meta_id = effect.relation.id
+        def_name = effect.relation.name
+
     eqs.append(body)
     new_body = mk_and(eqs)
-
-    is_output = isinstance(effect, ir.Output)
-    meta_id = effect.id if is_output else effect.relation.id
-    def_name = ctx.def_names.get_name_by_id(meta_id, "output") if is_output else effect.relation.name
     rel_id = get_relation_id(ctx, def_name, meta_id)
 
     # Context bookkeeping
-    if is_output:
+    if isinstance(effect, ir.Output):
         ctx.output_ids.append((rel_id, def_name))
 
     return lqp.Def(
@@ -112,6 +117,23 @@ def _translate_effect(ctx: TranslationCtx, effect: Union[ir.Output, ir.Update],
         meta = None,
     )
 
+def _translate_output_bindings(ctx: TranslationCtx, bindings: list[ir.Value]) -> Tuple[list[Tuple[lqp.Var, lqp.RelType]], list[lqp.Formula], str]:
+    symbol_literals = []
+    non_symbols = []
+    for binding in bindings:
+        if isinstance(binding, ir.Literal) and binding.type == types.Symbol:
+            symbol_literals.append(binding.value)
+        else:
+            non_symbols.append(binding)
+    projection, eqs = _translate_bindings(ctx, non_symbols)
+    if len(symbol_literals) > 0:
+        name_suffix = "_"
+        name_suffix += "_".join(symbol_literals)
+    else:
+        name_suffix = ""
+
+    return projection, eqs, name_suffix
+
 def _translate_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.Formula) -> lqp.Formula:
     # Ascending rank is constructed using rel_primitive_sort. If a limit is added to an
     # ascending rank we can use rel_primitive_top for an efficient evaluation.
@@ -129,9 +151,9 @@ def _translate_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.Formula) -> lq
         raise Exception("Mixed orderings in rank are not supported yet.")
 
     # Filter out the group-by variables, since they are introduced outside the rank.
-    input_args, input_eqs = translate_bindings(ctx, list(rank.args))
+    input_args, input_eqs = _translate_bindings(ctx, list(rank.args))
     introduced_meta_projs = [arg for arg in rank.projection if arg not in rank.group and arg not in rank.args]
-    projected_args, projected_eqs = translate_bindings(ctx, list(introduced_meta_projs))
+    projected_args, projected_eqs = _translate_bindings(ctx, list(introduced_meta_projs))
 
     body = mk_and([body] + input_eqs + projected_eqs)
     abstr_args = input_args + projected_args
@@ -263,12 +285,12 @@ def _translate_aggregate(ctx: TranslationCtx, aggr: ir.Aggregate, body: lqp.Form
     output_vars = [_translate_term(ctx, term)[0] for term in meta_output_terms]
 
     body_conjs = [body]
-    input_args, input_eqs = translate_bindings(ctx, meta_input_terms)
+    input_args, input_eqs = _translate_bindings(ctx, meta_input_terms)
 
     # Filter out the group-by variables, since they are introduced outside the aggregation.
     # Input terms are added later below.
     introduced_meta_projs = [arg for arg in aggr.projection if arg not in aggr.group and arg not in meta_input_terms]
-    projected_args, projected_eqs = translate_bindings(ctx, list(introduced_meta_projs))
+    projected_args, projected_eqs = _translate_bindings(ctx, list(introduced_meta_projs))
     body_conjs.extend(input_eqs)
     body_conjs.extend(projected_eqs)
     abstr_args: list[Tuple[lqp.Var, lqp.RelType]] = projected_args + input_args
@@ -332,7 +354,7 @@ def _translate_to_formula(ctx: TranslationCtx, task: ir.Task) -> lqp.Formula:
     elif isinstance(task, ir.Not):
         return lqp.Not(arg=_translate_to_formula(ctx, task.task), meta=None)
     elif isinstance(task, ir.Exists):
-        lqp_vars, conjuncts = translate_bindings(ctx, list(task.vars))
+        lqp_vars, conjuncts = _translate_bindings(ctx, list(task.vars))
         conjuncts.append(_translate_to_formula(ctx, task.task))
         return mk_exists(lqp_vars, mk_and(conjuncts))
     elif isinstance(task, ir.Construct):
@@ -463,7 +485,7 @@ def get_relation_id(ctx: TranslationCtx, orig_name: str, metamodel_id: int) -> l
     ctx.rel_id_to_orig_name[relation_id] = unique_name
     return relation_id
 
-def translate_bindings(ctx: TranslationCtx, bindings: list[ir.Value]) -> Tuple[list[Tuple[lqp.Var, lqp.RelType]], list[lqp.Formula]]:
+def _translate_bindings(ctx: TranslationCtx, bindings: list[ir.Value]) -> Tuple[list[Tuple[lqp.Var, lqp.RelType]], list[lqp.Formula]]:
     lqp_vars = []
     conjuncts = []
     for binding in bindings:
@@ -480,11 +502,7 @@ def binding_to_lqp_var(ctx: TranslationCtx, binding: ir.Value) -> Tuple[lqp.Var,
         assert isinstance(var, lqp.Var)
         return var, typ, None
     elif isinstance(binding, ir.Literal):
-        # HACK: unpack any symbol literals and just use the value.
-        if binding.type == types.Symbol:
-            return constant_to_lqp_var(ctx, binding.value, None)
-        else:
-            return constant_to_lqp_var(ctx, binding.value, binding.type)
+        return constant_to_lqp_var(ctx, binding.value, binding.type)
     elif isinstance(binding, lqp.PrimitiveValue):
         return constant_to_lqp_var(ctx, binding, None)
     else:
