@@ -14,6 +14,7 @@ from copy import copy
 from itertools import islice
 from datetime import datetime, date
 from decimal import Decimal as PyDecimal
+import numpy as np
 
 from .util import FrozenOrderedSet, Printer as BasePrinter, NameCache
 
@@ -800,7 +801,12 @@ class Printer(BasePrinter):
         elif isinstance(node, Data):
             length = len(node)
             vals = islice(node, min(length, 3))
-            vals_str = ", ".join([f"({', '.join(map(self.value_to_string, v))})" for v in vals])
+            def unwrap_val(v):
+                if isinstance(v, (np.integer, np.floating)):
+                    # convert DataFrame number types to native Python types
+                    v = v.item()
+                return self.value_to_string(v)
+            vals_str = ", ".join([f"({', '.join(map(unwrap_val, v))})" for v in vals])
             if length > 3:
                 vals_str += f", ... ({length} total)"
             vars_str = ", ".join([self.value_to_string(v) for v in node.vars])
