@@ -44,6 +44,9 @@ class SolverModel:
         declare {self.Constraint._name}
 
         declare {prefix_l}variable_name
+        declare {prefix_l}variable_lower
+        declare {prefix_l}variable_upper
+        declare {prefix_l}variable_fixed
         declare {prefix_l}minobjective_name
         declare {prefix_l}maxobjective_name
         declare {prefix_l}constraint_name
