@@ -522,13 +522,14 @@ class UnknownSourceWarning(RAIWarning):
 
         if len(source_fqns) == 1:
             msg = textwrap.fill(
-                f"Unable to access source '{source_fqns[0]}'. Ensure the source's fully qualified name is spelled correctly and that the role your program is executing with has permission to access it.",
+                f"Unable to access source '{source_fqns[0]}'. Ensure the fully qualified name is correct "
+                f"and that your active role(s) have the required privileges.",
                 78,
             )
         else:
             prefix = "The following sources could not be accessed:"
             suffix = textwrap.fill(
-                "Ensure the sources' fully qualified names are spelled correctly and that the role your program is executing with has permission to access them.",
+                "Ensure the fully qualified names are correct and that your active role(s) have the required privileges.",
                 78,
             )
             msg = "\n".join(
@@ -536,14 +537,13 @@ class UnknownSourceWarning(RAIWarning):
             )
 
         if role:
-            role_note = textwrap.fill(
-                f"Your current primary role '{role}' must be granted the required privileges. "
-                f"Due to a Snowflake secondary role issue, the RAI Python API only uses the primary role. "
-                f"You may have access in other clients if privileges are granted through secondary roles, "
-                f"but the RAI Python API will fail unless the primary role has the necessary permissions.",
-                78,
-            )
-            msg += "\n\n" + role_note
+            msg += f"\n\nYour current primary role is '{role}', and by default all assigned roles are used as secondary roles."
+
+        msg += "\n\n" + textwrap.fill(
+            "While access via secondary roles is supported, we recommend granting the necessary privileges "
+            "directly to your primary role to ensure consistent access.",
+            78,
+        )
 
         super().__init__(name, name, msg)
         self.source_fqns = source_fqns
@@ -1497,29 +1497,21 @@ class SnowflakeAppMissingException(RAIException):
         )
         self.name = "Couldn't find RelationalAI Snowflake application"
         self.content = self.format_message()
-
         super().__init__(self.message, self.name, self.content)
 
     def format_message(self):
-        base = textwrap.dedent(f"""
-            Either the app '{self.app_name}' isn't installed in this Snowflake account or you don't have permission to access it.
-
-            If it's installed under a different name, run 'rai init' on the command line to set the app name.
-        """)
-
+        base = (
+            f"Either the app '{self.app_name}' isn't installed in this Snowflake account, "
+            f"or your active role(s) don't have permission to access it.\n\n"
+            f"If it's installed under a different name, run 'rai init' on the command line to set the app name."
+        )
+        
         if self.role:
-            role_note = textwrap.dedent(f"""
-                If the app is installed but access is denied, check your current primary role: '{self.role}'.
-                Due to a Snowflake secondary role issue, the RAI Python API currently only allows access via the primary role.
-                You may have access in other clients if privileges are granted through secondary roles,
-                but the RAI Python API will fail unless the primary role has the required permissions.
-            """)
-        else:
-            role_note = textwrap.dedent("""
-                If the app is installed but your current role doesn't have permission to access the app, switch to a role that does.
-            """)
-
-        return base + "\n" + role_note
+            base += f"\n\nYour current primary role is '{self.role}', and by default all assigned roles are used as secondary roles."
+        
+        base += "\n\nWhile access via secondary roles is supported, we recommend granting the necessary privileges directly to your primary role to ensure consistent access."
+        
+        return base
 
 
 class SnowflakeImportMissingException(RAIException):
