@@ -256,7 +256,7 @@ def span_code(span:SpanNode):
 
 @ui.refreshable
 def details():
-    with ui.column().style("gap:2px; margin-left: 220px;"):
+    with ui.column().style("gap:2px; margin-left: 230px;"):
         for root in current_json_objects:
             span_code(root)
 
@@ -309,19 +309,19 @@ def span_ui(span: SpanNode, is_pass: bool=False):
             ui.label(name).style("padding:0; margin:0; color: #DDD;")
             ui.space()
             ui.label(f"{to_ms(span.elapsed)}").style("padding-right: 5px; color:#999;")
-            if is_pass:
-                me.on("click", lambda: jump_to_pass())
-                me.style("cursor: pointer;")
-            elif span.type == "create_v2":
+            if span.type == "create_v2":
                 txn_id = span.end_attrs.get("txn_id")
                 me.on("click", lambda: ui.navigate.to(f"https://171608476159.observeinc.com/workspace/41759331/log-explorer?datasetId=41832558&filter-rai_transaction_id={txn_id}", new_tab=True))
                 me.style("cursor: pointer;")
+            elif is_pass:
+                me.on("click", lambda: jump_to_pass())
+                me.style("cursor: pointer;")
         handle_body(span)
 
 
 @ui.refreshable
 def sidebar():
-    with ui.column().style("min-width: 250px; max-width: 300px; margin-top: 50px; height: calc(100vh - 60px); overflow-x: hidden; overflow-y: auto; position: fixed; top: 0; left: 0; z-index: 1000;"):
+    with ui.column().style("min-width: 270px; max-width: 270px; margin-top: 50px; height: calc(100vh - 60px); overflow-x: hidden; overflow-y: auto; position: fixed; top: 0; left: 0; z-index: 1000;"):
         for root in current_json_objects:
             span_ui(root)
 
